
subroutine CalcEnergyForces(Pos, T, k1, k2, Ext, Gamma, Alpha, Ksi, Lambda, Lc, PEnergy, Forces, Dim, NAtom)
	implicit none
	integer, intent(in) :: Dim, NAtom
	real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
	real(8), intent(in) :: T, k1, k2, Gamma, Alpha, Ksi, Lambda, Lc
    real(8), intent(in), dimension(Dim) :: Ext
	real(8), intent(out) :: PEnergy
	real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
	real(8), dimension(Dim) :: rij, rij2, Posi, Fij, PosStart
	real(8) :: r0, sd2, d2, sd22, d22, Theta, Theta0
	integer :: i, j
	PEnergy = 0.
	Forces = 0.
	r0 = Lc / (NAtom - 1)
!Tether the first atom to its initial position
	if (k1 > 0) then
		PosStart = (/32.8, 16.4, 32.8/)
		rij = Pos(0,:) - PosStart
		d2 = sum(rij*rij)
		PEnergy = PEnergy + (k1/2. * d2)
		Fij = rij * k1
		Forces(0,:) = Forces(0,:) - Fij
	endif
!Tether the last atom to a spring for umbrella sampling
    	if (k2 > 0.) then
		PosStart = Ext
        	rij = PosStart - Pos(NAtom-1,:)
		d2 = sum(rij*rij)
		PEnergy = PEnergy + (k2/2. * d2)
		Fij = rij * k2
		Forces(NAtom-1,:) = Forces(NAtom-1,:) + Fij
	endif
	do i = 0, NAtom - 2
		Posi = Pos(i,:)
!Calculate nearest neighbor potentials and forces
		rij = Pos(i+1,:) - Posi
		rij2 = Pos(i-1,:) - Posi
		d2 = sum(rij*rij)
		sd2 = sqrt(d2)
		d22 = sum(rij2*rij2)
		sd22 = sqrt(d22)
!Bond Potential
		PEnergy = PEnergy + ((Gamma/2.) * ((sd2 - r0)**2))
		Fij = rij * Gamma * (1 - (r0/sd2))
		Forces(i,:) = Forces(i,:) + Fij
		Forces(i+1,:) = Forces(i+1,:) - Fij
!Bend Potential
		if (i > 0) then
			Theta = acos(sum(rij2*rij)/(sd2*sd22))
			Theta0 = acos(-1.)
			PEnergy = PEnergy + ((Alpha/2.) * ((Theta - Theta0)**2))
			Forces(i-1,:) = Forces(i-1,:) + (Alpha*(Theta-Theta0)/sin(Theta))*(rij/(sd2*sd22) - cos(Theta)*rij2/d22)
			Forces(i+1,:) = Forces(i+1,:) + (Alpha*(Theta-Theta0)/sin(Theta))*(rij2/(sd2*sd22) - cos(Theta)*rij/d2)
		endif
!Coulomb Potential
		PEnergy = PEnergy + (Ksi * exp(-sd2/Lambda)/sd2)
		Fij = -(rij * Ksi * (Lambda + sd2) * exp(-sd2/Lambda)/(Lambda * sd2 * d2))
		Forces(i,:) = Forces(i,:) + Fij
		Forces(i+1,:) = Forces(i+1,:) - Fij
!Calculate long range potentials
		do j = i + 2, NAtom - 1	    
			rij = Pos(j,:) - Posi
			d2 = sum(rij * rij)
			sd2 = sqrt(d2)
			PEnergy = PEnergy + (Ksi * exp(-sd2/Lambda)/sd2)
			Fij = -(rij * Ksi * (Lambda + sd2) * exp(-sd2/Lambda)/(Lambda * sd2 * d2))
			Forces(i,:) = Forces(i,:) + Fij
			Forces(j,:) = Forces(j,:) - Fij
		enddo
	enddo
!PEnergy in Joules
	PEnergy = PEnergy * T / 1000000.
!Forces in Newtons
	Forces = Forces * T
end subroutine


subroutine VVIntegrate(Pos, T, k1, k2, Ext, Vel, Accel, Gamma, Alpha, Ksi, Lambda, Lc, dt, KEnergy, PEnergy, Dim, NAtom)
	implicit none
	integer, intent(in) :: Dim, NAtom
	real(8), intent(in) :: T, k1, k2, Gamma, Alpha, Ksi, Lambda, Lc, dt
    real(8), intent(in), dimension(Dim) :: Ext
	real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
	real(8), intent(out) :: KEnergy, PEnergy
	real(8) :: m
	external :: CalcEnergyForces
	m = (1.047e-19)/NAtom
	Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
	Vel = Vel + 0.5 * dt * Accel
	call CalcEnergyForces(Pos, T, k1, k2, Ext, Gamma, Alpha, Ksi, Lambda, Lc, PEnergy, Accel, Dim, NAtom)
!Acceleration in um/s2
	Accel = Accel * 1000000./m
!Velocity in m/s
	Vel = Vel + 0.5 * dt * Accel
!KEnergy in Joules
	KEnergy = 0.5 * m * sum(Vel*Vel)/1000000000000.
end subroutine